%define anolis_release 1
%bcond_with fulltest

# We need to manipulate the built *.so.%%sover
%global sover .2.1.0
%global soshort .2

Name:		blis
Version:	0.7.0
Release:	%{anolis_release}%{?dist}
Summary:	BLAS-like Library Instantiation Software Framework
License:	BSD
URL:		https://github.com/flame/blis
Source0:	https://github.com/flame/blis/archive/%version/%name-%version.tar.gz
BuildRequires:	perl
BuildRequires:	gcc
BuildRequires:	/usr/bin/python3 gcc-gfortran chrpath
BuildRequires:	make

%global desc \
BLIS is a portable software framework for instantiating\
high-performance BLAS-like dense linear algebra libraries.  The\
framework was designed to isolate essential kernels of computation\
that, when optimized, immediately enable optimized implementations of\
most of its commonly used and computationally intensive operations.\
While BLIS exports a new BLAS-like API, it also includes a BLAS\
compatibility layer which gives application developers access to BLIS\
implementations via traditional BLAS routine calls.\
\
This packaging contains automatically-dispatched\
architecture-optimized kernels for some targets, notably recent x86_64.

%description
%desc

This is the serial version.


%package	devel
Summary:	Development files for %name
Requires:	%name = %version-%release
Requires:	%name-openmp = %version-%release
Requires:	%name-threads = %version-%release
Requires:	%name-serial64 = %version-%release
Requires:	%name-openmp64 = %version-%release
Requires:	%name-threads64 = %version-%release

%description	devel
The %name-devel package contains libraries and header files for
developing applications that use %name.

%package serial64
Summary:	BLAS-like Library Instantiation Software Framework - 64-bit

%description serial64
%desc

This is the serial version with a 64-bit integer interface.

%package openmp
Summary:	BLAS-like Library Instantiation Software Framework - OpenMP

%description openmp
%desc

This is the OpenMP-parallelized version.

%package openmp64
Summary:	BLAS-like Library Instantiation Software Framework - OpenMP, 64-bit

%description openmp64
%desc

This is the OpenMP-parallelized version with a 64-bit integer interface.

# A pthreads version is necessary for Python (numpy) according to
# Debian openblas.
%package threads
Summary:	BLAS-like Library Instantiation Software Framework - pthreads

%description threads
%desc

This is the pthreads-parallelized version.

%package threads64
Summary:	BLAS-like Library Instantiation Software Framework - pthreads, 64-bit

%description threads64
%desc

This is the pthreads-parallelized version with a 64-bit integer interface.

%package srpm-macros
Summary:	BLIS architecture macros
BuildArch:	noarch

%description srpm-macros
BLIS architecture macros.

%package doc
Summary:        Documentation files for %{name}
Requires:       %{name} = %{EVR}
BuildArch:      noarch

%description    doc
The %{name}-doc package contains documentation files for %{name}.


%prep
%autosetup -n %{name}-%{version} -p1
echo %sover | awk -F. '{printf("%s\n%s.%s\n", $2,$3,$4)}' >so_version

%build
case %_arch in
x86_64) arch=x86_64 ;;
# a57 runs on all aarch64 and the optimized micro-kernel should be a
# better default than generic.
# Fixme:  Include my changes for arm and ppc micro-arch dispatch.
aarch64) arch=cortexa57 ;;
*) arch=generic ;;
esac

%global confflags --enable-debug=opt --disable-static --enable-shared --enable-verbose-make --enable-cblas
export CFLAGS="$RPM_OPT_FLAGS -O3 -funsafe-math-optimizations" LDFLAGS="%{?__global_ldflags}"
export PYTHON=python3		# Needed by both configure and make

# It's not an autotools configure
./configure --prefix=$(pwd)/o %confflags -t openmp $arch
%make_build SOFLAGS="-shared -Wl,-soname=libbliso.so%sover"
make install

./configure --prefix=$(pwd)/p %confflags -t pthreads $arch
%make_build SOFLAGS="-shared -Wl,-soname=libblisp.so%sover"
make install

# Rename the libraries per soname and generate BLAS_compatible ones
mkdir -p blisblas{,o,p,64,o64,p64}
for d in o p; do
  cd $d/lib
  f=libblis.so%sover
  mv $f ${f/./$d.}
  ln -s libblis$d.so%sover libblis$d.so
  ln -s libblis$d.so%sover  libblis$d.so%soshort
  rm libblis.*
  cd ../..
  cc -shared -Wl,-soname=libblas.so.3 -L$(pwd)/$d/lib -lblis$d -o blisblas$d/libblas.so.3 $LDFLAGS
  ln -s libblas.so.3 blisblas$d/libblas.so
done
./configure --prefix=$(pwd)/64 %confflags -b 64 $arch
%make_build SOFLAGS="-shared -Wl,-soname=libblis64.so%sover"
make install

./configure --prefix=$(pwd)/o64 %confflags -b 64 -t openmp $arch
%make_build SOFLAGS="-shared -Wl,-soname=libbliso64.so%sover"
make install

./configure --prefix=$(pwd)/p64 %confflags -b 64 -t pthreads $arch
%make_build SOFLAGS="-shared -Wl,-soname=libblisp64.so%sover"
make install

for d in 64 o64 p64; do
  cd $d/lib
  f=libblis.so%sover
  mv $f ${f/./$d.}
  ln -s libblis$d.so%sover libblis$d.so
  ln -s libblis$d.so%sover libblis$d.so%soshort
  rm -f libblis.*
  cd ../..
  cc -shared -Wl,-soname=libblas64.so.3 -L$(pwd)/$d/lib -lblis$d -o blisblas$d/libblas64.so.3 $LDFLAGS
  ln -s libblas64.so.3 blisblas$d/libblas64.so
done

# done last for the benefit of check
./configure --prefix=$(pwd)/serial %confflags $arch
%make_build
make install
cc -shared -Wl,-soname=libblas.so.3 -L$(pwd)/serial/lib -lblis -o blisblas/libblas.so.3 $LDFLAGS
ln -s libblas.so.3 blisblas/libblas.so


%install
mkdir -p %buildroot%_libdir %buildroot%_includedir

cp -a {serial,o,p}/lib/* %buildroot%_libdir
mv serial/include/blis %buildroot%_includedir
for d in o p; do
  cp -a $d/include/blis %buildroot%_includedir/blis$d
done
cp -a {64,o64,p64}/lib/* %buildroot%_libdir
for d in 64 o64 p64; do
  cp -a $d/include/blis %buildroot%_includedir/blis$d
done
# Needed for debuginfo processing
chmod +x %buildroot%_libdir/*.so.*
cp -a blisblas* %buildroot%_libdir
# This is quite large.
gzip CHANGELOG
chrpath -d %buildroot%_libdir/*.so.*

%global macrosdir %(d=%{_rpmconfigdir}/macros.d; [ -d $d ] || d=%{_sysconfdir}/rpm; echo $d)
mkdir -p %buildroot%macrosdir
cat <<EOF >%buildroot%macrosdir/macros.blis-srpm
# Architectures for for BLIS has an optimized implementation
%blis_opt_arches x86_64 aarch64
EOF

%generate_compatibility_deps


%check
%{?dts:. /opt/rh/devtoolset-%{?dts}/enable}
# A quick check which tests the Fortran BLAS interface with gfortran,
# unlike the "test" or "check" targets.
# Fixme: check a 64-bit version where relevant
gfortran -o dblat blastest/src/fortran/dblat3.f -Lblisblas -Lserial/lib -lblas -lblis
LD_LIBRARY_PATH=$(pwd)/serial/lib:$(pwd)/blisblas ./dblat <<+ || { cat dblat3.summ && false; }
'dblat3.summ'
6
'dblat3.snap'
-1
F
T
T
16.0
7
0 1 2 3 7 31 63
3
0.0 1.0 0.7
3
0.0 1.0 1.3
DGEMM  T
DSYMM  F
+

export LD_LIBRARY_PATH=`pwd`/serial/lib
%if %{with fulltest}
%make_build test
%else
%make_build check
%endif



%files
%license LICENSE
%{_libdir}/libblis.so%{soshort}*
%{_libdir}/blisblas
%dir %{abidir}
%{abidir}/libblis.dump

%files openmp
%license LICENSE
%{_libdir}/libbliso.so%{soshort}*
%{_libdir}/blisblaso
%{abidir}/libbliso.dump

%files threads
%license LICENSE
%{_libdir}/libblisp.so%{soshort}*
%{_libdir}/blisblasp
%{abidir}/libblisp.dump

%files serial64
%license LICENSE
%{_libdir}/libblis64.so%{soshort}*
%{_libdir}/blisblas64
%{abidir}/libblis64.dump

%files openmp64
%license LICENSE
%{_libdir}/libbliso64.so%{soshort}*
%{_libdir}/blisblaso64
%{abidir}/libbliso64.dump

%files threads64
%license LICENSE
%{_libdir}/libblisp64.so%{soshort}*
%{_libdir}/blisblasp64
%{abidir}/libblisp64.dump

%files devel
%doc examples
%{_includedir}/*
%{_libdir}/libblis*.so

%files srpm-macros
%{macrosdir}/macros.blis-srpm

%files doc
%doc CHANGELOG.gz CREDITS README.md

%changelog
* Sun Mar 26 2023 happy_orange <songnannan@linux.alibaba.com> - 0.7.0-1
- init package from upstream
